%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% DemoConductivityMUSIC
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Overview:
%   Direct imaging of small particles for the Helmholtz equation
%
% References:
%   Section 8.5
%   Mathematical and Computational Methods in Photonics and Phononics
%
% Authors:
%   Habib Ammari, Brian Fitzpatrick, Hyeonbae Kang,
%   Matias Ruiz, Sanghyeon Yu, Hai Zhang
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear
set(0,'DefaultFigureWindowStyle','docked')

geom = @circleg;  % circle geometry
[p,e,t] = initmesh(geom);  %coarse mesh (inverse problem)
[p1,e1,t1] = refinemesh(geom,p,e,t);  % fine mesh (direct problem)

figure
subplot(1,2,1)
pdemesh(p,e,t)
subplot(1,2,2)
pdemesh(p1,e1,t1)

nt = size(t,2); % Number of triangles of coarse mesh
bound = p(:,e(1,:)); %boundary points

% Find centroids and areas of triangles of coarse mesh
x = (p(1,t(1,:))+p(1,t(2,:))+p(1,t(3,:)))/3;
y = (p(2,t(1,:))+p(2,t(2,:))+p(2,t(3,:)))/3;
area = pdetrg(p,t);

% The real conductivity
sigma = coeffunction2(p1,t1);
sigmamesh = tri2grid2(p1,t1,sigma,x,y);

% considered range size for the response matrix
rangeA = 4;

n = 30;
noise = 0.000; % Noise level, in percentage

Theta = linspace(0,2*pi,n+1);
Theta = Theta(1:end-1);
Ubound = cell(1,n);
ubound = cell(1,n);

for l=1:n
    y = 1.3*[cos(Theta(l)); sin(Theta(l))];
    gradw = @(x) 1/(2*pi)/norm(x-y)^2*(x-y);

    for i=1:size(bound,2)
        dwdn{l}(1,i) = dot(gradw(bound(:,i)),bound(:,i));
    end
    f = @(theta) -1/(2*pi)*(dot(y,[cos(theta); sin(theta)])-1)/norm(y-[cos(theta); sin(theta)]).^2;
    
    neumann = @(p,e,u,time) pdeboundeit(p,e,u,time,f);
    u = assempde(neumann,p1,e1,t1,sigma,0,0); %real potential
    U = assempde(neumann,p1,e1,t1,1,0,0); % reference potential

    % boundary measurements
    for i=1:size(bound,2)
        Ubound{l}(1,i) = tri2grid(p1,t1,U,bound(1,i),bound(2,i));
        ubound{l}(1,i) = tri2grid(p1,t1,u,bound(1,i),bound(2,i));
    end
   
    ubound{l} = ubound{l} + noise*randn(size(ubound{l}))*max(abs(ubound{l}));
end

%% MUSIC
A = zeros(n,n);
for l=1:n
    for lp=1:n
        A(l,lp) = sum(dwdn{l}.*(ubound{lp}-Ubound{lp}))/size(bound,2)*2*pi;
    end
end

[U,S,V] = svd(A);
Q = V(:,1:rangeA);
P = Q*Q';
Pnoise = eye(n) - P;

% Find centroids and areas of triangles of fine mesh
zsx = (p1(1,t1(1,:))+p1(1,t1(2,:))+p1(1,t1(3,:)))/3;
zsy = (p1(2,t1(1,:))+p1(2,t1(2,:))+p1(2,t1(3,:)))/3;

IMU = zeros(1,length(zsx));
G = zeros(2,n);

for i=1:length(zsx)
    for l = 1:n
        y = 1.2*[cos(Theta(l)); sin(Theta(l))];
        gradw = @(x) 1/(2*pi)/norm(x-y)^2*(x-y);
        G(:,l) = gradw([zsx(i);zsy(i)]);
    end
    
    IMU(i) = (norm(Pnoise*G(1,:)')^2+norm(Pnoise*G(2,:)')^2)^(-1/2);
end

figure
subplot(1,2,1)
pdeplot(p1,e1,t1,'xydata',sigma)
title('Real Conductivity')
axis equal tight
subplot(1,2,2)
pdeplot(p1,e1,t1,'xydata',IMU)
title('Reconstructed inclusion location')
axis equal tight
    