%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% DemoHelmholtzAnomalyImaging
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Overview:
%   Direct imaging of small particles for the Helmholtz equation
%
% References:
%   Section 8.5
%   Mathematical and Computational Methods in Photonics and Phononics
%
% Authors:
%   Habib Ammari, Brian Fitzpatrick, Hyeonbae Kang,
%   Matias Ruiz, Sanghyeon Yu, Hai Zhang
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear
set(0,'DefaultFigureWindowStyle','docked')

geom = @circleg;  % circle geometry
[p,e,t] = initmesh(geom);  %coarse mesh (inverse problem)
[p1,e1,t1] = refinemesh(geom,p,e,t);  % fine mesh (direct problem)
[p2,e2,t2] = refinemesh(geom,p1,e1,t1);
[p1,e1,t1] = refinemesh(geom,p2,e2,t2);

figure
subplot(1,2,1)
pdemesh(p,e,t)
subplot(1,2,2)
pdemesh(p1,e1,t1)

nt = size(t,2); % Number of triangles of coarse mesh
bound = p2(:,e2(1,:)); %boundary points

% The real conductivity
a = mu(p1,t1);
epsi = epsilon(p1,t1);
%epsi = 1;

% Wave frequency
k0=12;

% Considered rango of the response matrix for MUSIC

rangeA=12;

n = 30;
noise = 0.000; % Noise level, in percentage

Theta = linspace(0,2*pi,n+1);
Theta = Theta(1:end-1);
Ubound = cell(1,n);
ubound = cell(1,n);
U = zeros(size(a));

figure
parfor l=1:n
    theta = [cos(Theta(l)); sin(Theta(l))];
    gradw = @(x) -1i*k0*theta*exp(-1i*k0*dot(theta,x));
    gradw2= @(x) 1i*k0*theta*exp(1i*k0*dot(theta,x));
    
    for i=1:size(bound,2)
        dwdn{l}(i,1) = dot(conj(gradw(bound(:,i))),bound(:,i));
        dwdn2{l}(i,1) = dot(conj(gradw2(bound(:,i))),bound(:,i));
    end
    f = @(t)  (1i*k0*dot(theta,[cos(t);sin(t)])*exp(1i*k0*dot(theta,[cos(t);sin(t)])));
    
    neumann = @(p,e,u,time) pdebound(p,e,u,time,f);
    u = assempde(neumann,p1,e1,t1,a,-k0^2*epsi,0); %real potential
    U = assempde(neumann,p1,e1,t1,1,-k0^2,0); % reference potential
    
    % boundary measurements
    for i=1:size(bound,2)
        Ubound{l}(i,1) = tri2grid(p1,t1,U,bound(1,i),bound(2,i));
        ubound{l}(i,1) = tri2grid(p1,t1,u,bound(1,i),bound(2,i));
    end
   
    ubound{l} = ubound{l} + noise*randn(size(ubound{l}))*max(abs(ubound{l}));
end

%% 2. MUSIC, Back-propagation and Kirkhoff
display('A')
A = zeros(n,n);
A2= zeros(n,n);
parfor l=1:n
    for lp=1:n
        A(l,lp) = sum(dwdn{lp}.*(ubound{l}-Ubound{l}))/size(bound,2)*2*pi;
        A2(l,lp) = sum(dwdn2{lp}.*(ubound{l}-Ubound{l}))/size(bound,2)*2*pi;
    end
end

display('music')
[~,S,V] = svd(A);
Q = V(:,1:rangeA);
P = Q*Q';
Pnoise = eye(n) - P;

% Find centroids and areas of triangles of coarse mesh
zsx = (p2(1,t2(1,:))+p2(1,t2(2,:))+p2(1,t2(3,:)))/3;
zsy = (p2(2,t2(1,:))+p2(2,t2(2,:))+p2(2,t2(3,:)))/3;

display('BP and KM')

IMU = zeros(1,length(zsx));
IKM = zeros(1,length(zsx));
G = cell(1,length(zsx));

parfor i=1:length(zsx)
    G{i} = zeros(3,n);
    for l = 1:n
        theta = [cos(Theta(l)); sin(Theta(l))];
        gg = @(x) theta*exp(1i*k0*dot(theta,x));
        G{i}(1:2,l) = gg([zsx(i);zsy(i)])/(sqrt(n));
        G{i}(3,l) = exp(1i*k0*dot(theta,[zsx(i);zsy(i)]))/sqrt(n);      
    end

    IMU(i) = (norm(Pnoise*G{i}(1,:).')^2+norm(Pnoise*G{i}(2,:).')^2+norm(Pnoise*G{i}(3,:).')^2)^(-1/2);
    IKM(i) = dot((G{i}(1,:)),A*G{i}(1,:).') + dot((G{i}(2,:)),A*G{i}(2,:).') + dot((G{i}(3,:)),A*G{i}(3,:).');
end
    
figure
subplot(2,3,1)
pdeplot(p1,e1,t1,'xydata',a)
% title('Magnetic coefficient')
subplot(2,3,2)
pdeplot(p1,e1,t1,'xydata',epsi)
% title('Electrical coefficient')
subplot(2,3,3)
pdeplot(p2,e2,t2,'xydata',IMU)
% title('MUSIC reconstruction')   
subplot(2,3,4)
pdeplot(p2,e2,t2,'xydata',real(IKM))
% title('Kirchhoff migration')